@@ -1,6 +1,4 @@
-"""
-A subfolder for area management
-"""
+"""A submodule with for implementations of area managers."""
 
 from .batch import BatchAreaManager
 from .custom_grid import CustomGridAreaManager
