@@ -1,6 +1,5 @@
-"""
-Module with utilities for logging
-"""
+"""Implementation of LoggingManager and different handlers used for logging."""
+import contextlib
 import json
 import logging
 import sys
@@ -13,9 +12,9 @@ from fs.base import FS
 from fs.errors import FilesystemClosed
 from pydantic import Field
 
-from eolearn.core.utils.fs import join_path
-from sentinelhub import SHConfig
+from eolearn.core.utils.fs import join_path, unpickle_fs
 
+from ..utils.eopatch_list import save_names
 from ..utils.fs import LocalFile
 from ..utils.general import jsonify
 from ..utils.logging import get_instance_info
@@ -30,7 +29,6 @@ class LoggingManager(EOGrowObject):
     """A class that manages logging specifics"""
 
     class Schema(ManagerSchema):
-
         save_logs: bool = Field(
             False,
             description=(
@@ -221,18 +219,16 @@ class LoggingManager(EOGrowObject):
         with self.storage.filesystem.open(report_filename, "w") as report_file:
             json.dump(report, report_file, indent=2, default=jsonify)
 
-    def save_eopatch_execution_status(self, pipeline_execution_name: str, finished: list, failed: list) -> None:
+    def save_eopatch_execution_status(
+        self, pipeline_execution_name: str, finished: List[str], failed: List[str]
+    ) -> None:
         """Saves lists of EOPatch names for which execution either finished successfully or failed"""
         if not self.config.save_logs:
             return
 
-        filesystem = self.storage.filesystem
         logs_folder = self.get_pipeline_logs_folder(pipeline_execution_name)
-
         for eopatches, filename in [(finished, "finished.json"), (failed, "failed.json")]:
-            path = fs.path.combine(logs_folder, filename)
-            with filesystem.open(path, "w") as file:
-                json.dump(eopatches, file, indent=2)
+            save_names(self.storage.filesystem, fs.path.combine(logs_folder, filename), eopatches)
 
 
 class FilesystemHandler(FileHandler):
@@ -247,38 +243,40 @@ class FilesystemHandler(FileHandler):
     process stuck waiting for a thread lock release.
     """
 
-    def __init__(self, path: str, filesystem: Optional[FS] = None, config: Optional[SHConfig] = None, **kwargs: Any):
+    def __init__(self, path: str, filesystem: Union[FS, bytes], encoding: Optional[str] = "utf-8", **kwargs: Any):
         """
-        :param path: A path to a log file. It should be an absolute path if filesystem object is not given and relative
-            otherwise.
-        :param filesystem: A filesystem to where logs will be written.
-        :param config: A config object holding credentials.
+        :param path: A path to a log file that is relative to the given `filesystem` object.
+        :param filesystem: A filesystem to where logs will be written. It can either be an instance of a filesystem
+            object or its pickled copy.
+        :param encoding: Encoding used to write log files.
         :param kwargs: Keyword arguments that will be propagated to FileHandler.
         """
-        self.local_file = LocalFile(path, mode="w", filesystem=filesystem, config=config)
+        filesystem_object = unpickle_fs(filesystem) if isinstance(filesystem, bytes) else filesystem
+        self.local_file = LocalFile(path, mode="w", filesystem=filesystem_object)
 
-        super().__init__(self.local_file.path, **kwargs)
+        super().__init__(self.local_file.path, encoding=encoding, **kwargs)
 
         self.addFilter(FilesystemFilter())
 
     def close(self) -> None:
         """Closes logging and closes the local file"""
         super().close()
-        try:
+        with contextlib.suppress(FilesystemClosed):
             self.local_file.close()
-        except FilesystemClosed:
-            pass
 
 
 class RegularBackupHandler(FilesystemHandler):
     """A customized FilesystemHandler that makes a copy to a remote location regularly after given amount of time."""
 
-    def __init__(self, *args: Any, backup_interval: Union[float, int], **kwargs: Any):
+    def __init__(self, path: str, filesystem: Union[FS, bytes], backup_interval: Union[float, int], **kwargs: Any):
         """
+        :param path: A path to a log file that is relative to the given `filesystem` object.
+        :param filesystem: A filesystem to where logs will be written. It can either be an instance of a filesystem
+            object or its pickled copy.
         :param backup_interval: A minimal number of seconds before handler will back up the log file to the remote
             location. The backup will only happen when the next log record will be emitted.
         """
-        super().__init__(*args, **kwargs)
+        super().__init__(path=path, filesystem=filesystem, **kwargs)
 
         self.backup_interval = backup_interval
         self._last_backup_time = time.monotonic()
@@ -387,6 +385,8 @@ class EOExecutionFilter(Filter):
         "rasterio",
         "numba",
         "fiona.ogrext",
+        "fiona.env",
+        "fiona._env",
     )
 
     def __init__(self, ignore_packages: Optional[Sequence[str]] = None, *args: Any, **kwargs: Any):
