@@ -1,7 +1,5 @@
-"""
-Conversion of batch results to EOPatches
-"""
-from typing import Any, Dict, List, Optional
+"""Pipeline for conversion of batch results to EOPatches."""
+from typing import Any, List, Optional
 
 import numpy as np
 from pydantic import Field, root_validator
@@ -14,6 +12,7 @@ from eolearn.core import (
     MergeFeatureTask,
     OverwritePermission,
     RemoveFeatureTask,
+    RenameFeatureTask,
     SaveTask,
 )
 from eolearn.features import LinearFunctionTask
@@ -22,9 +21,9 @@ from eolearn.io import ImportFromTiffTask
 from ..core.pipeline import Pipeline
 from ..core.schemas import BaseSchema
 from ..tasks.batch_to_eopatch import DeleteFilesTask, FixImportedTimeDependentFeatureTask, LoadUserDataTask
+from ..types import ExecKwargs, Feature, FeatureSpec, PatchList, RawSchemaDict
 from ..utils.filter import get_patches_with_missing_features
-from ..utils.types import Feature, FeatureSpec
-from ..utils.validators import field_validator, optional_field_validator, parse_dtype
+from ..utils.validators import optional_field_validator, parse_dtype
 
 
 class FeatureMappingSchema(BaseSchema):
@@ -40,32 +39,16 @@ class FeatureMappingSchema(BaseSchema):
     multiply_factor: float = Field(1, description="Factor used to multiply feature values with.")
     dtype: Optional[np.dtype] = Field(description="Dtype of the output feature.")
     _parse_dtype = optional_field_validator("dtype", parse_dtype, pre=True)
-    save_early: bool = Field(
-        True,
-        description=(
-            "The pipeline allows that custom processing steps are added at the end. If this parameter is set to True"
-            " then the feature will be saved and removed from memory immediately after being loaded from tiffs. This"
-            " makes the pipeline more memory efficient. If set to False it will keep the feature in memory until the"
-            " end so that it can be used for processing steps."
-        ),
-    )
-
-
-def _sort_feature_mappings(mapping: List[FeatureMappingSchema]) -> List[FeatureMappingSchema]:
-    """Sorts feature mappings by putting the ones that will be saved early at the beginning. This optimizes the memory
-    usage."""
-    return sorted(mapping, key=lambda feature_mapping: not feature_mapping.save_early)
 
 
 class BatchToEOPatchPipeline(Pipeline):
     class Schema(Pipeline.Schema):
         input_folder_key: str = Field(description="Storage manager key pointing to the path with Batch results")
-        output_folder_key: str = Field(description="Storage manager key pointing to where the eopatches are saved")
+        output_folder_key: str = Field(description="Storage manager key pointing to where the EOPatches are saved")
 
         mapping: List[FeatureMappingSchema] = Field(
             description="A list of mapping from batch files into EOPatch features."
         )
-        _sort_mapping = field_validator("mapping", _sort_feature_mappings)
 
         userdata_feature_name: Optional[str] = Field(
             description="A name of META_INFO feature in which userdata.json would be stored."
@@ -82,7 +65,7 @@ class BatchToEOPatchPipeline(Pipeline):
         )
 
         @root_validator
-        def check_something_is_converted(cls, values):  # type: ignore[no-untyped-def]
+        def check_something_is_converted(cls, values: RawSchemaDict) -> RawSchemaDict:
             """Check that the pipeline has something to do."""
             params = "userdata_feature_name", "userdata_timestamp_reader", "mapping"
             assert any(
@@ -96,11 +79,11 @@ class BatchToEOPatchPipeline(Pipeline):
         """Additionally sets some basic parameters calculated from config parameters"""
         super().__init__(*args, **kwargs)
 
-        self._input_folder = self.storage.get_folder(self.config.input_folder_key, full_path=True)
+        self._input_folder = self.storage.get_folder(self.config.input_folder_key)
         self._has_userdata = self.config.userdata_feature_name or self.config.userdata_timestamp_reader
         self._all_batch_files = self._get_all_batch_files()
 
-    def filter_patch_list(self, patch_list: List[str]) -> List[str]:
+    def filter_patch_list(self, patch_list: PatchList) -> PatchList:
         """EOPatches are filtered according to existence of specified output features"""
 
         filtered_patch_list = get_patches_with_missing_features(
@@ -125,67 +108,52 @@ class BatchToEOPatchPipeline(Pipeline):
 
         return features
 
-    def _get_final_output_features(self) -> List[FeatureSpec]:
-        """Output features that will be saved in the final saving step and not sooner."""
-        output_features = self._get_output_features()
-        features_saved_early = {
-            feature_mapping.feature for feature_mapping in self.config.mapping if feature_mapping.save_early
-        }
-        return [feature for feature in output_features if feature not in features_saved_early]
-
     def build_workflow(self) -> EOWorkflow:
         """Builds the workflow"""
-        previous_node = None
+        userdata_node = None
         if self._has_userdata:
-            previous_node = EONode(
+            userdata_node = EONode(
                 LoadUserDataTask(
                     path=self._input_folder,
+                    filesystem=self.storage.filesystem,
                     userdata_feature_name=self.config.userdata_feature_name,
                     userdata_timestamp_reader=self.config.userdata_timestamp_reader,
-                    config=self.sh_config,
                 )
             )
 
-        for feature_mapping in self.config.mapping:
-            feature = feature_mapping.feature
-            mapping_node = self._get_tiff_mapping_node(feature_mapping, previous_node)
+        mapping_nodes = [
+            self._get_tiff_mapping_node(feature_mapping, userdata_node) for feature_mapping in self.config.mapping
+        ]
 
-            if feature_mapping.save_early:
-                save_task = SaveTask(
-                    path=self.storage.get_folder(self.config.output_folder_key, full_path=True),
-                    features=[feature],
-                    compress_level=1,
-                    overwrite_permission=OverwritePermission.OVERWRITE_FEATURES,
-                    config=self.sh_config,
-                )
-                save_node = EONode(save_task, inputs=[mapping_node])
+        last_node = userdata_node
+        if len(mapping_nodes) == 1:
+            last_node = mapping_nodes[0]
+        elif len(mapping_nodes) > 1:
+            last_node = EONode(MergeEOPatchesTask(), inputs=mapping_nodes)
 
-                _, f_name = feature
-                previous_node = EONode(RemoveFeatureTask([feature]), inputs=[save_node], name=f"Remove {f_name}")
-            else:
-                previous_node = mapping_node
-
-        if previous_node is None:
+        if last_node is None:
             raise ValueError(
                 "At least one of `userdata_feature_name`, `userdata_timestamp_reader`, or `mapping` has to be set in"
                 " the config. This should have been caught in the validation phase, please report issue."
             )
 
-        processing_node = self.get_processing_node(previous_node)
+        processing_node = self.get_processing_node(last_node)
 
         save_task = SaveTask(
-            path=self.storage.get_folder(self.config.output_folder_key, full_path=True),
-            features=self._get_final_output_features(),
+            path=self.storage.get_folder(self.config.output_folder_key),
+            filesystem=self.storage.filesystem,
+            features=self._get_output_features(),
             compress_level=1,
             overwrite_permission=OverwritePermission.OVERWRITE_FEATURES,
-            config=self.sh_config,
         )
         save_node = EONode(save_task, inputs=([processing_node] if processing_node else []))
 
         cleanup_node = None
         if self.config.remove_batch_data:
             delete_task = DeleteFilesTask(
-                path=self._input_folder, filenames=self._all_batch_files, config=self.sh_config
+                path=self._input_folder,
+                filesystem=self.storage.filesystem,
+                filenames=self._all_batch_files,
             )
             cleanup_node = EONode(delete_task, inputs=[save_node], name="Delete batch data")
 
@@ -210,7 +178,11 @@ class BatchToEOPatchPipeline(Pipeline):
                 FeatureType.MASK_TIMELESS if feature_type.is_discrete() else FeatureType.DATA_TIMELESS
             ), feature[1]
 
-            import_task = ImportFromTiffTask(tmp_timeless_feature, folder=self._input_folder, config=self.sh_config)
+            import_task = ImportFromTiffTask(
+                tmp_timeless_feature,
+                folder=self._input_folder,
+                filesystem=self.storage.filesystem,
+            )
             # Filename is written into the dependency name to be used later for execution arguments:
             import_node = EONode(
                 import_task,
@@ -227,10 +199,7 @@ class BatchToEOPatchPipeline(Pipeline):
         previous_node = EONode(MergeEOPatchesTask(), inputs=end_nodes) if len(end_nodes) > 1 else end_nodes[0]
 
         final_feature = feature_type, feature_name
-        merge_feature_task = MergeFeatureTask(input_features=tmp_features, output_feature=final_feature)
-        merge_node = EONode(merge_feature_task, inputs=[previous_node])
-
-        end_node = EONode(RemoveFeatureTask(tmp_features), inputs=[merge_node])
+        end_node = self._get_feature_merge_node(previous_node, tmp_features, final_feature)
 
         if mapping.multiply_factor != 1 or mapping.dtype is not None:
             multiply_task = LinearFunctionTask(final_feature, slope=mapping.multiply_factor, dtype=mapping.dtype)
@@ -238,28 +207,48 @@ class BatchToEOPatchPipeline(Pipeline):
 
         return end_node
 
+    @staticmethod
+    def _get_feature_merge_node(
+        previous_node: EONode, input_features: List[Feature], output_feature: Feature
+    ) -> EONode:
+        """Merges input features into a single output feature and removes input features. In case there is a single
+        input feature this method just renames it into the output feature. This way it avoids memory duplication that
+        otherwise happens in `MergeFeatureTask`."""
+        if len(input_features) == 1:
+            feature_type, input_feature_name = input_features[0]
+            _, output_feature_name = output_feature
+
+            rename_task = RenameFeatureTask([(feature_type, input_feature_name, output_feature_name)])
+            return EONode(rename_task, inputs=[previous_node])
+
+        merge_feature_task = MergeFeatureTask(input_features=input_features, output_feature=output_feature)
+        merge_node = EONode(merge_feature_task, inputs=[previous_node])
+
+        remove_task = RemoveFeatureTask(input_features)
+        return EONode(remove_task, inputs=[merge_node])
+
     @staticmethod
     def get_processing_node(previous_node: EONode) -> EONode:
         """This method can be overwritten to add more tasks that process loaded data before saving it."""
         return previous_node
 
-    def get_execution_arguments(self, workflow: EOWorkflow) -> List[Dict[EONode, Dict[str, object]]]:
+    def get_execution_arguments(self, workflow: EOWorkflow, patch_list: PatchList) -> ExecKwargs:
         """Prepare execution arguments per each EOPatch"""
-        exec_args = super().get_execution_arguments(workflow)
+        exec_args = super().get_execution_arguments(workflow, patch_list)
 
         nodes = workflow.get_nodes()
 
-        for name, single_exec_dict in zip(self.patch_list, exec_args):
+        for patch_name, patch_args in exec_args.items():
             for node in nodes:
                 if isinstance(node.task, ImportFromTiffTask):
                     if node.name is None:
                         raise RuntimeError("One of the ImportFromTiffTask nodes has not been tagged with filename.")
                     filename = node.name.split()[0]
-                    path = f"{name}/{filename}"
-                    single_exec_dict[node] = dict(filename=path)
+                    path = f"{patch_name}/{filename}"
+                    patch_args[node] = dict(filename=path)
 
                 if isinstance(node.task, (DeleteFilesTask, LoadUserDataTask)):
-                    single_exec_dict[node] = dict(folder=name)
+                    patch_args[node] = dict(folder=patch_name)
 
         return exec_args
 
