@@ -1,11 +1,9 @@
-"""
-Module implementing pipelines for downloading data
-"""
+"""Implements different customizeable pipelines for downloading data."""
 import abc
 import datetime as dt
 import logging
 from contextlib import nullcontext
-from typing import Any, Callable, Dict, List, Optional, Tuple, Union
+from typing import Any, Callable, List, Optional, Tuple, Union
 
 import numpy as np
 import ray
@@ -28,9 +26,10 @@ from sentinelhub.download import SessionSharing, collect_shared_session
 
 from ..core.pipeline import Pipeline
 from ..core.schemas import BaseSchema
+from ..types import ExecKwargs, Feature, FeatureSpec, PatchList, ProcessingType, TimePeriod
 from ..utils.filter import get_patches_with_missing_features
-from ..utils.types import Feature, FeatureSpec, Path, ProcessingType, TimePeriod
 from ..utils.validators import (
+    ensure_exactly_one_defined,
     field_validator,
     optional_field_validator,
     parse_data_collection,
@@ -94,7 +93,7 @@ class BaseDownloadPipeline(Pipeline, metaclass=abc.ABCMeta):
         super().__init__(*args, **kwargs)
         self.download_node_uid: Optional[str] = None
 
-    def filter_patch_list(self, patch_list: List[str]) -> List[str]:
+    def filter_patch_list(self, patch_list: PatchList) -> PatchList:
         """EOPatches are filtered according to existence of specified output features"""
 
         filtered_patch_list = get_patches_with_missing_features(
@@ -146,34 +145,32 @@ class BaseDownloadPipeline(Pipeline, metaclass=abc.ABCMeta):
 
         end_node = EONode(
             SaveTask(
-                self.storage.get_folder(self.config.output_folder_key, full_path=True),
+                self.storage.get_folder(self.config.output_folder_key),
+                filesystem=self.storage.filesystem,
                 features=self._get_output_features(),
                 compress_level=self.config.compress_level,
                 overwrite_permission=OverwritePermission.OVERWRITE_FEATURES,
-                config=self.sh_config,
             ),
             inputs=[postprocessing_node or download_node],
         )
 
         return EOWorkflow.from_endnodes(end_node)
 
-    def get_execution_arguments(self, workflow: EOWorkflow) -> List[Dict[EONode, Dict[str, object]]]:
+    def get_execution_arguments(self, workflow: EOWorkflow, patch_list: PatchList) -> ExecKwargs:
         """Adds required bbox and time_interval parameters for input task to the base execution arguments
 
         :param workflow: EOWorkflow used to download images
         """
-        exec_args = super().get_execution_arguments(workflow)
+        exec_args = super().get_execution_arguments(workflow, patch_list)
 
         download_node = workflow.get_node_with_uid(self.download_node_uid)
         if download_node is None:
             return exec_args
 
-        bbox_list = self.eopatch_manager.get_bboxes(eopatch_list=self.patch_list)
-
-        for index, bbox in enumerate(bbox_list):
-            exec_args[index][download_node] = {"bbox": bbox}
+        for patch_name, bbox in patch_list:
+            exec_args[patch_name][download_node] = {"bbox": bbox}
             if hasattr(self.config, "time_period"):
-                exec_args[index][download_node]["time_interval"] = self.config.time_period  # type: ignore[attr-defined]
+                exec_args[patch_name][download_node]["time_interval"] = self.config.time_period
 
         return exec_args
 
@@ -181,8 +178,9 @@ class BaseDownloadPipeline(Pipeline, metaclass=abc.ABCMeta):
         execution_kind = self._init_processing()
         session_loader = self._create_session_loader(execution_kind)
 
+        patch_list = self.get_patch_list()
         workflow = self.build_workflow(session_loader)
-        exec_args = self.get_execution_arguments(workflow)
+        exec_args = self.get_execution_arguments(workflow, patch_list)
 
         context: Union[SessionSharing, nullcontext] = nullcontext()
         if execution_kind is ProcessingType.MULTI:
@@ -194,10 +192,26 @@ class BaseDownloadPipeline(Pipeline, metaclass=abc.ABCMeta):
 
 
 class CommonDownloadFields(BaseSchema):
-    data_collection: DataCollection = Field(description="Data collection from which data will be downloaded.")
+    data_collection: DataCollection = Field(
+        description=(
+            "Data collection from which data will be downloaded. See `utils.validators.parse_data_collection` for more"
+            " info on input options."
+        )
+    )
     _validate_data_collection = field_validator("data_collection", parse_data_collection, pre=True)
 
-    resolution: float = Field(description="Resolution of downloaded data in meters")
+    resolution: Optional[float] = Field(
+        description=(
+            "Resolution of downloaded data in meters. Exactly one of the parameters resolution and size has to be"
+            " specified."
+        )
+    )
+    size: Optional[Tuple[int, int]] = Field(
+        description=(
+            "A pair (width, height) of downloaded data in pixels. . Exactly one of the parameters resolution and size"
+            " has to be specified."
+        )
+    )
 
     maxcc: Optional[float] = Field(ge=0, le=1, description="Maximal cloud coverage filter.")
 
@@ -205,6 +219,8 @@ class CommonDownloadFields(BaseSchema):
         description="A type of downsampling and upsampling used by Sentinel Hub service. Default is NEAREST"
     )
 
+    _check_resolution_and_size = ensure_exactly_one_defined("resolution", "size")
+
 
 class TimeDependantFields(BaseSchema):
     time_period: TimePeriod
@@ -258,6 +274,7 @@ class DownloadPipeline(BaseDownloadPipeline):
             bands_feature=(FeatureType.DATA, self.config.bands_feature_name),
             bands=self.config.bands,
             resolution=self.config.resolution,
+            size=self.config.size,
             maxcc=self.config.maxcc,
             time_difference=time_diff,
             data_collection=data_collection,
@@ -266,7 +283,8 @@ class DownloadPipeline(BaseDownloadPipeline):
             bands_dtype=bands_dtype,
             config=self.sh_config,
             mosaicking_order=self.config.mosaicking_order,
-            aux_request_args=_get_aux_request_args(self.config.resampling_type),
+            downsampling=self.config.resampling_type,
+            upsampling=self.config.resampling_type,
             session_loader=session_loader,
         )
         return EONode(download_task)
@@ -277,7 +295,7 @@ class DownloadEvalscriptPipeline(BaseDownloadPipeline):
 
     class Schema(BaseDownloadPipeline.Schema, CommonDownloadFields, TimeDependantFields):
         features: List[Feature] = Field(description="Features to construct from the evalscript")
-        evalscript_path: Path
+        evalscript_path: str
 
     config: Schema
 
@@ -295,12 +313,14 @@ class DownloadEvalscriptPipeline(BaseDownloadPipeline):
             evalscript=evalscript,
             data_collection=self.config.data_collection,
             resolution=self.config.resolution,
+            size=self.config.size,
             maxcc=self.config.maxcc,
             time_difference=time_diff,
             max_threads=self.config.threads_per_worker,
             config=self.sh_config,
             mosaicking_order=self.config.mosaicking_order,
-            aux_request_args=_get_aux_request_args(self.config.resampling_type),
+            downsampling=self.config.resampling_type,
+            upsampling=self.config.resampling_type,
             session_loader=session_loader,
         )
         return EONode(download_task)
@@ -318,21 +338,16 @@ class DownloadTimelessPipeline(BaseDownloadPipeline):
         return [(FeatureType.DATA_TIMELESS, self.config.feature_name), FeatureType.BBOX]
 
     def _get_download_node(self, session_loader: SessionLoaderType) -> EONode:
-
         download_task = SentinelHubDemTask(
             feature=(FeatureType.DATA_TIMELESS, self.config.feature_name),
             data_collection=self.config.data_collection,
             resolution=self.config.resolution,
+            size=self.config.size,
             maxcc=self.config.maxcc,
             max_threads=self.config.threads_per_worker,
             config=self.sh_config,
-            aux_request_args=_get_aux_request_args(self.config.resampling_type),
+            downsampling=self.config.resampling_type,
+            upsampling=self.config.resampling_type,
             session_loader=session_loader,
         )
         return EONode(download_task)
-
-
-def _get_aux_request_args(resampling: Optional[ResamplingType]) -> Optional[dict]:
-    if resampling is not None:
-        return {"processing": {"downsampling": resampling, "upsampling": resampling}}
-    return None
