@@ -1,6 +1,4 @@
-"""
-Download pipeline that works with Sentinel Hub batch service
-"""
+"""Download pipeline that works with Sentinel Hub batch service."""
 import logging
 from typing import Any, DefaultDict, Dict, List, Literal, Optional, Tuple
 
@@ -25,7 +23,7 @@ from sentinelhub import (
 from ..core.area.batch import BatchAreaManager
 from ..core.pipeline import Pipeline
 from ..core.schemas import BaseSchema
-from ..utils.types import Path, TimePeriod
+from ..types import TimePeriod
 from ..utils.validators import field_validator, optional_field_validator, parse_data_collection, parse_time_period
 
 LOGGER = logging.getLogger(__name__)
@@ -34,7 +32,12 @@ LOGGER = logging.getLogger(__name__)
 class InputDataSchema(BaseSchema):
     """Parameter structure for a single data collection used in a batch request."""
 
-    data_collection: DataCollection = Field(description="Data collection from which data will be downloaded.")
+    data_collection: DataCollection = Field(
+        description=(
+            "Data collection from which data will be downloaded. See `utils.validators.parse_data_collection` for more"
+            " info on input options."
+        )
+    )
     _validate_data_collection = field_validator("data_collection", parse_data_collection, pre=True)
 
     time_period: Optional[TimePeriod]
@@ -64,7 +67,7 @@ class BatchDownloadPipeline(Pipeline):
         )
 
         inputs: List[InputDataSchema]
-        evalscript_path: Path
+        evalscript_path: str
 
         tiff_outputs: List[str] = Field(default_factory=list, description="Names of TIFF outputs of a batch job")
         save_userdata: bool = Field(
@@ -73,7 +76,7 @@ class BatchDownloadPipeline(Pipeline):
         batch_output_kwargs: dict = Field(
             default_factory=dict,
             description=(
-                "Any other arguments to be added to a dictionary of parameters. Passed as **kwargs to the `output`"
+                "Any other arguments to be added to a dictionary of parameters. Passed as `**kwargs` to the `output`"
                 " method of `SentinelHubBatch` during the creation process."
             ),
         )
@@ -121,15 +124,10 @@ class BatchDownloadPipeline(Pipeline):
 
         self.batch_client = SentinelHubBatch(config=self.sh_config)
 
-        if self.area_manager.subsplit != (1, 1):
-            raise ValueError(
-                f"Cannot run {self.__class__.__name__} if {self.area_manager.__class__.__name__} is using a subsplit."
-                " Set subsplit parameters to 1."
-            )
-
     def run_procedure(self) -> Tuple[List[str], List[str]]:
         """Procedure that uses Sentinel Hub batch service to download data to an S3 bucket."""
         batch_request = self._create_or_collect_batch_request()
+
         user_action = self._trigger_user_action(batch_request)
 
         if user_action is BatchUserAction.ANALYSE or (
@@ -142,7 +140,7 @@ class BatchDownloadPipeline(Pipeline):
                 sleep_time=self.config.monitoring_analysis_sleep_time,
             )
 
-        self.ensure_batch_grid(batch_request.request_id)
+        self.cache_batch_area_manager_grid(batch_request.request_id)
 
         if self.config.analysis_only:
             return [], []
@@ -157,6 +155,9 @@ class BatchDownloadPipeline(Pipeline):
 
         processed = self._get_tile_names_from_results(results, BatchTileStatus.PROCESSED)
         failed = self._get_tile_names_from_results(results, BatchTileStatus.FAILED)
+        log_msg = f"Successfully downloaded {len(processed)} tiles"
+        log_msg += f", but {len(failed)} tiles failed." if failed else "."
+        LOGGER.info(log_msg)
         return processed, failed
 
     def _create_or_collect_batch_request(self) -> BatchRequest:
@@ -246,17 +247,17 @@ class BatchDownloadPipeline(Pipeline):
         LOGGER.info("Didn't trigger batch job because current batch request status is %s", status)
         return BatchUserAction.NONE
 
-    def ensure_batch_grid(self, request_id: str) -> None:
+    def cache_batch_area_manager_grid(self, request_id: str) -> None:
         """This method ensures that area manager caches batch grid into the storage."""
-        if self.area_manager.batch_id and self.area_manager.batch_id != request_id:
+        if self.area_manager.config.batch_id and self.area_manager.config.batch_id != request_id:
             raise ValueError(
                 f"{self.area_manager.__class__.__name__} is set to use batch request with ID "
-                f"{self.area_manager.batch_id} but {self.__class__.__name__} is using batch request with ID "
+                f"{self.area_manager.config.batch_id} but {self.__class__.__name__} is using batch request with ID "
                 f"{request_id}. Make sure that you use the same IDs."
             )
-        self.area_manager.batch_id = request_id
+        self.area_manager._injected_batch_id = request_id
 
-        self.area_manager.cache_grid()
+        self.area_manager.get_grid()  # this caches the grid for later use
 
     @staticmethod
     def _get_tile_names_from_results(
