@@ -1,6 +1,4 @@
-"""
-A pipeline to construct features for training/prediction
-"""
+"""Implements a pipeline to construct features for training/prediction."""
 import logging
 from typing import Dict, List, Optional, Tuple
 
@@ -29,15 +27,14 @@ from ..tasks.features import (
     ValidDataFractionPredicate,
     join_valid_and_cloud_masks,
 )
+from ..types import Feature, FeatureSpec, PatchList, TimePeriod
 from ..utils.filter import get_patches_with_missing_features
-from ..utils.types import Feature, FeatureSpec, TimePeriod
 from ..utils.validators import field_validator, optional_field_validator, parse_dtype, parse_time_period
 
 LOGGER = logging.getLogger(__name__)
 
 
 class ValidityFiltering(BaseSchema):
-
     cloud_mask_feature_name: Optional[str] = Field(
         description="Name of cloud mask to enable additional filtering by cloud"
     )
@@ -78,7 +75,7 @@ class FeaturesPipeline(Pipeline):
 
     config: Schema
 
-    def filter_patch_list(self, patch_list: List[str]) -> List[str]:
+    def filter_patch_list(self, patch_list: PatchList) -> PatchList:
         """EOPatches are filtered according to existence of specified output features"""
 
         filtered_patch_list = get_patches_with_missing_features(
@@ -115,11 +112,11 @@ class FeaturesPipeline(Pipeline):
         postprocessing_node = self.get_postprocessing_node(ndi_node)
 
         save_task = SaveTask(
-            self.storage.get_folder(self.config.output_folder_key, full_path=True),
+            self.storage.get_folder(self.config.output_folder_key),
+            filesystem=self.storage.filesystem,
             features=self._get_output_features(),
             compress_level=self.config.compress_level,
             overwrite_permission=OverwritePermission.OVERWRITE_FEATURES,
-            config=self.sh_config,
         )
         save_node = EONode(save_task, inputs=[postprocessing_node])
 
@@ -133,9 +130,9 @@ class FeaturesPipeline(Pipeline):
         filtering_config = self.config.data_preparation
 
         load_task = LoadTask(
-            self.storage.get_folder(self.config.input_folder_key, full_path=True),
+            self.storage.get_folder(self.config.input_folder_key),
+            filesystem=self.storage.filesystem,
             lazy_loading=True,
-            config=self.sh_config,
         )
         end_node = EONode(load_task)
 
@@ -192,7 +189,9 @@ class InterpolationFeaturesPipeline(FeaturesPipeline):
 
     class Schema(FeaturesPipeline.Schema):
         interpolation: Optional[InterpolationSpecifications] = Field(
-            "Fine-tuning of interpolation parameters. If not set, the interpolation will work on current timestamps"
+            description=(
+                "Fine-tuning of interpolation parameters. If not set, the interpolation will work on current timestamps"
+            )
         )
 
     config: Schema
@@ -234,7 +233,9 @@ class MosaickingFeaturesPipeline(FeaturesPipeline):
 
     class Schema(FeaturesPipeline.Schema):
         mosaicking: MosaickingSpecifications = Field(
-            "Fine-tuning of mosaicking parameters. If not set, the interpolation will work on current timestamps"
+            description=(
+                "Fine-tuning of mosaicking parameters. If not set, the interpolation will work on current timestamps"
+            )
         )
 
     config: Schema
