@@ -1,6 +1,4 @@
-"""
-Module implementing merge samples
-"""
+"""Implements a pipeline for merging sampled features into numpy arrays fit for training models."""
 import logging
 from typing import List, Literal, Optional, Tuple, cast
 
@@ -12,7 +10,7 @@ from eolearn.core import EOPatch, EOWorkflow, FeatureType, LoadTask, OutputTask,
 from eolearn.core.utils.fs import get_full_path
 
 from ..core.pipeline import Pipeline
-from ..utils.types import Feature, FeatureSpec
+from ..types import Feature, FeatureSpec
 
 LOGGER = logging.getLogger(__name__)
 
@@ -31,7 +29,12 @@ class MergeSamplesPipeline(Pipeline):
             description="Dictionary of all features for which samples are to be merged."
         )
         include_timestamp: bool = Field(False, description="Whether to also prepare an array of merged timestamps.")
-        id_filename: Optional[str] = Field(description="Filename of array holding patch id of concatenated features")
+        id_filename: Optional[str] = Field(
+            description=(
+                "Filename of array holding patch ID of concatenated features. The patch ID is the index of the patch in"
+                " the final patch list, any filtration of the patch list will impact the results."
+            )
+        )
         suffix: str = Field("", description="String to append to array filenames")
         workers: int = Field(1, description="Number of threads used to load data from EOPatches in parallel.")
         use_ray: Literal[False] = Field(False, description="Pipeline does not parallelize properly.")
@@ -44,7 +47,8 @@ class MergeSamplesPipeline(Pipeline):
     def run_procedure(self) -> Tuple[List[str], List[str]]:
         """Procedure which merges data from EOPatches into ML-ready numpy arrays"""
         workflow = self.build_workflow()
-        exec_args = self.get_execution_arguments(workflow)
+        patch_list = self.get_patch_list()
+        exec_args = self.get_execution_arguments(workflow, patch_list)
 
         # It doesn't make sense to parallelize loading over a cluster, but it would # make sense to parallelize over
         # features that have to be concatenated or, if we would concatenate into multiple files, parallelize creating
@@ -62,9 +66,9 @@ class MergeSamplesPipeline(Pipeline):
         features_to_load: List[FeatureSpec] = [FeatureType.TIMESTAMP] if self.config.include_timestamp else []
         features_to_load.extend(self.config.features_to_merge)
         load_task = LoadTask(
-            self.storage.get_folder(self.config.input_folder_key, full_path=True),
+            self.storage.get_folder(self.config.input_folder_key),
+            filesystem=self.storage.filesystem,
             features=features_to_load,
-            config=self.sh_config,
         )
         output_task = OutputTask(name=self._OUTPUT_NAME)
         return EOWorkflow(linearly_connect_tasks(load_task, output_task))
@@ -93,17 +97,15 @@ class MergeSamplesPipeline(Pipeline):
         if self.config.include_timestamp:
             arrays = []
             for patch, sample_num in zip(patches, patch_sample_nums):
-                arrays.append(np.tile(patch.timestamp, (sample_num, 1)))
+                arrays.append(np.tile(np.array(patch.timestamp), (sample_num, 1)))
                 patch.timestamp = []
 
             self._save_array(np.concatenate(arrays, axis=0), "TIMESTAMPS")
 
         if self.config.id_filename:
             LOGGER.info("Started merging EOPatch ids")
-            patch_ids = self.eopatch_manager.get_id_list_from_eopatch_list(patch_names)
             patch_id_arrays = [
-                np.ones(sample_num, dtype=np.uint32) * patch_id
-                for sample_num, patch_id in zip(patch_sample_nums, patch_ids)
+                np.ones(sample_num, dtype=np.uint32) * patch_id for patch_id, sample_num in enumerate(patch_sample_nums)
             ]
 
             merged_patch_ids: np.ndarray = np.concatenate(patch_id_arrays, axis=0)
@@ -121,7 +123,7 @@ class MergeSamplesPipeline(Pipeline):
 
         del patch[feature]
 
-        axis = feature_type.ndim() - 2
+        axis = feature_type.ndim() - 2  # type: ignore[operator]
         feature_array = np.squeeze(feature_array, axis=axis)
 
         if feature_type in [FeatureType.DATA, FeatureType.MASK]:
