@@ -1,18 +1,17 @@
-"""
-Module implementing prediction pipeline
-"""
+"""Implements a base prediction pipeline and a LGBM specialized classification and regression pipelines."""
 import abc
 from typing import List, Optional, Tuple
 
+import fs
 import numpy as np
-from pydantic import Field, validator
+from pydantic import Field, root_validator
 
 from eolearn.core import EONode, EOWorkflow, FeatureType, LoadTask, MergeEOPatchesTask, OverwritePermission, SaveTask
 
 from ..core.pipeline import Pipeline
 from ..tasks.prediction import ClassificationPredictionTask, RegressionPredictionTask
+from ..types import Feature, FeatureSpec, PatchList, RawSchemaDict
 from ..utils.filter import get_patches_with_missing_features
-from ..utils.types import Feature, FeatureSpec
 from ..utils.validators import optional_field_validator, parse_dtype
 
 
@@ -43,17 +42,24 @@ class BasePredictionPipeline(Pipeline, metaclass=abc.ABCMeta):
             description="Name of `MASK_TIMELESS` feature which defines which areas will be predicted"
         )
 
-        @validator("prediction_mask_feature_name")
-        def validate_filename(cls, v, values):  # type: ignore[no-untyped-def]
-            if values["prediction_mask_folder_key"] is None:
-                assert v is None, "Both `prediction_mask_folder_key` and `prediction_mask_feature_name` must be given."
-            return v
-
         model_folder_key: str = Field(
             description="The storage manager key pointing to the folder of the model used in the prediction pipeline."
         )
         compress_level: int = Field(1, description="Level of compression used in saving EOPatches")
 
+        @root_validator
+        def validate_prediction_mask(cls, values: RawSchemaDict) -> RawSchemaDict:
+            """If prediction mask is defined then also its input folder has to be defined."""
+            is_mask_defined = values.get("prediction_mask_feature_name") is not None
+            is_folder_defined = values.get("prediction_mask_folder_key") is not None
+
+            if is_mask_defined:
+                assert (
+                    is_folder_defined
+                ), "Parameter prediction_mask_feature_name is defined but prediction_mask_folder_key is not."
+
+            return values
+
     config: Schema
 
     @abc.abstractmethod
@@ -65,7 +71,7 @@ class BasePredictionPipeline(Pipeline, metaclass=abc.ABCMeta):
         """If a multiprocessing lock is needed when executing"""
         return not self.config.use_ray and self.config.workers > 1
 
-    def filter_patch_list(self, patch_list: List[str]) -> List[str]:
+    def filter_patch_list(self, patch_list: PatchList) -> PatchList:
         """EOPatches are filtered according to existence of specified output features"""
 
         filtered_patch_list = get_patches_with_missing_features(
@@ -91,9 +97,9 @@ class BasePredictionPipeline(Pipeline, metaclass=abc.ABCMeta):
         """Returns nodes containing for loading and preparing the data as well as the endpoint tasks"""
         features_load_node = EONode(
             LoadTask(
-                self.storage.get_folder(self.config.input_folder_key, full_path=True),
+                self.storage.get_folder(self.config.input_folder_key),
+                filesystem=self.storage.filesystem,
                 features=[FeatureType.BBOX, FeatureType.TIMESTAMP, *self.config.input_features],
-                config=self.sh_config,
             )
         )
 
@@ -102,9 +108,9 @@ class BasePredictionPipeline(Pipeline, metaclass=abc.ABCMeta):
 
         mask_load_node = EONode(
             LoadTask(
-                self.storage.get_folder(self.config.prediction_mask_folder_key, full_path=True),
+                self.storage.get_folder(self.config.prediction_mask_folder_key),
+                filesystem=self.storage.filesystem,
                 features=[(FeatureType.MASK_TIMELESS, self.config.prediction_mask_feature_name)],
-                config=self.sh_config,
             )
         )
 
@@ -117,10 +123,10 @@ class BasePredictionPipeline(Pipeline, metaclass=abc.ABCMeta):
     def _get_saving_node(self, previous_node: EONode) -> EONode:
         """Returns nodes for finalizing and saving features"""
         save_task = SaveTask(
-            self.storage.get_folder(self.config.output_folder_key, full_path=True),
+            self.storage.get_folder(self.config.output_folder_key),
+            filesystem=self.storage.filesystem,
             features=self._get_output_features(),
             overwrite_permission=OverwritePermission.OVERWRITE_FEATURES,
-            config=self.sh_config,
             compress_level=self.config.compress_level,
         )
 
@@ -141,15 +147,15 @@ class RegressionPredictionPipeline(BasePredictionPipeline):
         return [FeatureType.BBOX, (FeatureType.DATA_TIMELESS, self.config.output_feature_name)]
 
     def _get_prediction_node(self, previous_node: EONode) -> EONode:
+        model_path = fs.path.join(self.storage.get_folder(self.config.model_folder_key), self.config.model_filename)
         prediction_task = RegressionPredictionTask(
-            model_folder=self.storage.get_folder(self.config.model_folder_key, full_path=True),
-            model_filename=self.config.model_filename,
+            model_path=model_path,
+            filesystem=self.storage.filesystem,
             input_features=self.config.input_features,
             mask_feature=_optional_typed_feature(FeatureType.MASK_TIMELESS, self.config.prediction_mask_feature_name),
             output_feature=(FeatureType.DATA_TIMELESS, self.config.output_feature_name),
             output_dtype=self.config.dtype,
             mp_lock=self._is_mp_lock_needed,
-            sh_config=self.sh_config,
             clip_predictions=self.config.clip_predictions,
         )
         return EONode(prediction_task, inputs=[previous_node])
@@ -176,9 +182,10 @@ class ClassificationPredictionPipeline(BasePredictionPipeline):
         return features
 
     def _get_prediction_node(self, previous_node: EONode) -> EONode:
+        model_path = fs.path.join(self.storage.get_folder(self.config.model_folder_key), self.config.model_filename)
         prediction_task = ClassificationPredictionTask(
-            model_folder=self.storage.get_folder(self.config.model_folder_key, full_path=True),
-            model_filename=self.config.model_filename,
+            model_path=model_path,
+            filesystem=self.storage.filesystem,
             input_features=self.config.input_features,
             mask_feature=_optional_typed_feature(FeatureType.MASK_TIMELESS, self.config.prediction_mask_feature_name),
             output_feature=(FeatureType.MASK_TIMELESS, self.config.output_feature_name),
@@ -187,7 +194,6 @@ class ClassificationPredictionPipeline(BasePredictionPipeline):
             ),
             output_dtype=self.config.dtype,
             mp_lock=self._is_mp_lock_needed,
-            sh_config=self.sh_config,
             label_encoder_filename=self.config.label_encoder_filename,
         )
         return EONode(prediction_task, inputs=[previous_node])
