@@ -1,6 +1,4 @@
-"""
-A pipeline module for rasterizing vector datasets.
-"""
+"""Implements a pipeline for rasterizing vector datasets."""
 import logging
 import os
 import uuid
@@ -10,7 +8,7 @@ import fiona
 import fs
 import geopandas as gpd
 import numpy as np
-from pydantic import Field, validator
+from pydantic import Field
 
 from eolearn.core import (
     CreateEOPatchTask,
@@ -22,16 +20,15 @@ from eolearn.core import (
     OverwritePermission,
     SaveTask,
 )
-from eolearn.core.utils.fs import join_path
 from eolearn.geometry import VectorToRasterTask
 from eolearn.io import VectorImportTask
 
 from ..core.pipeline import Pipeline
 from ..core.schemas import BaseSchema
+from ..types import Feature, FeatureSpec, PatchList
 from ..utils.filter import get_patches_with_missing_features
 from ..utils.fs import LocalFile
-from ..utils.types import Feature, FeatureSpec
-from ..utils.validators import field_validator, parse_dtype
+from ..utils.validators import ensure_exactly_one_defined, field_validator, parse_dtype
 from ..utils.vector import concat_gdf
 
 LOGGER = logging.getLogger(__name__)
@@ -50,17 +47,19 @@ class VectorColumnSchema(BaseSchema):
     )
     output_feature: Feature = Field(description="Output feature of rasterization.")
     polygon_buffer: float = Field(0, description="The size of polygon buffering to be applied before rasterization.")
-    resolution: float = Field(description="Rendering resolution in meters.")
+    resolution: Optional[float] = Field(
+        description="Rendering resolution in meters. Cannot be used with `raster_shape`."
+    )
+    raster_shape: Optional[Tuple[int, int]] = Field(
+        description="Shape of resulting raster image. Cannot be used with `resolution`."
+    )
     overlap_value: Optional[int] = Field(description="Value to write over the areas where polygons overlap.")
     dtype: np.dtype = Field(np.dtype("int32"), description="Numpy dtype of the output feature.")
     _parse_dtype = field_validator("dtype", parse_dtype, pre=True)
     no_data_value: int = Field(0, description="The no_data_value argument to be passed to VectorToRasterTask")
 
-    @validator("values_column")
-    def check_value_settings(cls, v, values):  # type: ignore[no-untyped-def]
-        """Ensures that precisely one of `value` and `values_column` is set."""
-        assert (v is None) != (values["value"] is None), "Only one of `values_column` and `value` should be given."
-        return v
+    _check_value_values_column = ensure_exactly_one_defined("value", "values_column")
+    _check_shape_resolution = ensure_exactly_one_defined("raster_shape", "resolution")
 
 
 class Preprocessing(BaseSchema):
@@ -95,6 +94,7 @@ class RasterizePipeline(Pipeline):
                 " step."
             )
         )
+        compress_level: int = Field(1, description="Level of compression used in saving EOPatches")
 
     config: Schema
 
@@ -114,7 +114,7 @@ class RasterizePipeline(Pipeline):
         else:
             self.vector_feature = self.config.vector_input
 
-    def filter_patch_list(self, patch_list: List[str]) -> List[str]:
+    def filter_patch_list(self, patch_list: PatchList) -> PatchList:
         filtered_patch_list = get_patches_with_missing_features(
             self.storage.filesystem,
             self.storage.get_folder(self.config.output_folder_key),
@@ -132,11 +132,12 @@ class RasterizePipeline(Pipeline):
     def run_dataset_preprocessing(self, filename: str, preprocess_config: Preprocessing) -> None:
         """Loads datasets, applies preprocessing steps and saves them to a cache folder"""
         LOGGER.info("Preprocessing dataset %s", filename)
+        gpd_engine = self.storage.config.geopandas_backend
 
         file_path = fs.path.combine(self.storage.get_input_data_folder(), filename)
         with LocalFile(file_path, mode="r", filesystem=self.storage.filesystem) as local_file:
             dataset_layers = [
-                gpd.read_file(local_file.path, layer=layer, encoding="utf-8")
+                gpd.read_file(local_file.path, layer=layer, encoding="utf-8", engine=gpd_engine)
                 for layer in fiona.listlayers(local_file.path)
             ]
 
@@ -144,9 +145,9 @@ class RasterizePipeline(Pipeline):
 
         dataset_gdf = self.preprocess_dataset(dataset_gdf)
 
-        dataset_path = self._get_dataset_path(filename, full_path=False)
+        dataset_path = self._get_dataset_path(filename)
         with LocalFile(dataset_path, mode="w", filesystem=self.storage.filesystem) as local_file:
-            dataset_gdf.to_file(local_file.path, encoding="utf-8", driver="GPKG")
+            dataset_gdf.to_file(local_file.path, encoding="utf-8", driver="GPKG", engine=gpd_engine)
 
     def build_workflow(self) -> EOWorkflow:
         """Creates workflow that is divided into the following sub-parts:
@@ -161,14 +162,17 @@ class RasterizePipeline(Pipeline):
             create_node = EONode(CreateEOPatchTask())
             path = self._get_dataset_path(self.filename)
             import_task = VectorImportTask(
-                self.vector_feature, path=path, layer=self.config.dataset_layer, config=self.sh_config
+                self.vector_feature,
+                path=path,
+                filesystem=self.storage.filesystem,
+                layer=self.config.dataset_layer,
             )
             data_preparation_node = EONode(import_task, inputs=[create_node])
         else:
             input_task = LoadTask(
-                self.storage.get_folder(self.config.input_folder_key, full_path=True),
+                self.storage.get_folder(self.config.input_folder_key),
+                filesystem=self.storage.filesystem,
                 features=[self.vector_feature, FeatureType.BBOX],
-                config=self.sh_config,
             )
             data_preparation_node = EONode(input_task)
 
@@ -179,10 +183,11 @@ class RasterizePipeline(Pipeline):
         postprocess_node = self.get_postrasterization_node(rasterization_node)
 
         save_task = SaveTask(
-            self.storage.get_folder(self.config.output_folder_key, full_path=True),
+            self.storage.get_folder(self.config.output_folder_key),
+            filesystem=self.storage.filesystem,
             features=self._get_output_features(),
             overwrite_permission=OverwritePermission.OVERWRITE_FEATURES,
-            config=self.sh_config,
+            compress_level=self.config.compress_level,
         )
         save_node = EONode(save_task, inputs=[postprocess_node])
 
@@ -209,6 +214,7 @@ class RasterizePipeline(Pipeline):
                     values=column.value,
                     buffer=column.polygon_buffer,
                     raster_resolution=column.resolution,
+                    raster_shape=column.raster_shape,
                     raster_dtype=np.dtype(column.dtype),
                     no_data_value=column.no_data_value,
                     overlap_value=column.overlap_value,
@@ -232,17 +238,15 @@ class RasterizePipeline(Pipeline):
             raise ValueError(f"Input file path {value} should be a GeoJSON, Shapefile, GeoPackage or GeoDataBase.")
         return value
 
-    def _get_dataset_path(self, filename: str, full_path: bool = True) -> str:
+    def _get_dataset_path(self, filename: str) -> str:
         """Provides a path from where dataset should be loaded into the workflow"""
         if self.config.preprocess_dataset is not None:
-            folder = self.storage.get_cache_folder(full_path=full_path)
+            folder = self.storage.get_cache_folder()
             filename = f"preprocessed_{filename}"
             filename = (os.path.splitext(filename))[0] + ".gpkg"
         else:
-            folder = self.storage.get_input_data_folder(full_path=full_path)
+            folder = self.storage.get_input_data_folder()
 
-        if full_path:
-            return join_path(folder, filename)
         return fs.path.combine(folder, filename)
 
     def _get_output_features(self) -> List[FeatureSpec]:
