@@ -1,6 +1,5 @@
-"""
-Module implementing pipelines for training an ML classifier
-"""
+"""Implements a base training pipeline and a LGBM specialized classification and regression model training pipelines."""
+
 import abc
 import logging
 from typing import Any, Dict, List, Literal, Optional, Tuple
@@ -178,7 +177,6 @@ class BaseTrainingPipeline(Pipeline, metaclass=abc.ABCMeta):
 
 
 class ClassificationPreprocessSchema(BaseSchema):
-
     label_encoder_filename: Optional[str] = Field(
         description="If specified uses a label encoder and saves it under specified name."
     )
@@ -191,7 +189,7 @@ class ClassificationPreprocessSchema(BaseSchema):
     )
 
 
-class ClassificationTrainingPipeline(BaseTrainingPipeline, metaclass=abc.ABCMeta):
+class ClassificationTrainingPipeline(BaseTrainingPipeline):
     """A base pipeline for training an ML classifier. Uses LGBMClassifier by default."""
 
     class Schema(BaseTrainingPipeline.Schema):
