@@ -4,17 +4,19 @@ A module containing general utilities that haven't been sorted in any other modu
 import datetime as dt
 import math
 from enum import Enum
-from typing import Tuple
+from typing import Tuple, Union
 
 import numpy as np
-from aenum import MultiValueEnum
 
 from sentinelhub import BBox, DataCollection
 from sentinelhub.data_collections import DataCollectionDefinition
 
 
-def jsonify(param: object) -> str:
+def jsonify(param: object) -> Union[str, list]:
     """Transforms an object into a normal string."""
+    if isinstance(param, set):
+        return list(param)
+
     if isinstance(param, (dt.datetime, dt.date)):
         return param.isoformat()
 
@@ -23,9 +25,6 @@ def jsonify(param: object) -> str:
     if isinstance(param, DataCollection):
         return param.name
 
-    if isinstance(param, MultiValueEnum):
-        return param.values[0]
-
     if isinstance(param, Enum):
         return param.value
 
